#!/usr/bin/perl
use strict;
use warnings;
use Test::More qw(no_plan);

use_ok( 'RDF::Query' );

my @files	= map { File::Spec->rel2abs( "data/$_" ) } qw(about.xrdf foaf.xrdf);
my @models;

{
	my @data		= map { RDF::Redland::URI->new( 'file://' . $_ ) } @files;
	my $storage		= new RDF::Redland::Storage("hashes", "test", "new='yes',hash-type='memory'");
	my $model		= new RDF::Redland::Model($storage, "");
	my $parser		= new RDF::Redland::Parser("rdfxml");
	$parser->parse_into_model($_, $_, $model) for (@data);
	my $bridge		= RDF::Query::Model::Redland->new( $model );
	isa_ok( $bridge->model, 'RDF::Redland::Model' );
	push(@models, $model);
} {
	my $storage		= new RDF::Core::Storage::Memory;
	my $model		= new RDF::Core::Model (Storage => $storage);
	foreach my $file (@files) {
		my $parser		= new RDF::Core::Model::Parser (
								Model			=> $model,
								Source			=> $file,
								SourceType		=> 'file',
								BaseURI			=> 'http://example.com/'
						);
		$parser->parse;
	}
	my $bridge		= RDF::Query::Model::RDFCore->new( $model );
	isa_ok( $bridge->model, 'RDF::Core::Model' );
	push(@models, $model);
}

foreach my $model (@models) {
	print "\n#################################\n";
	print "### Using model: $model\n\n";


	{
		print "# language typed literal\n";
		my $query	= new RDF::Query ( <<'END', undef, undef, 'sparql' );
			PREFIX	foaf: <http://xmlns.com/foaf/0.1/>
			SELECT	?person ?homepage
			WHERE	{
						?person foaf:name "Gary Peck"@en ; foaf:homepage ?homepage .
					}
END
		my $stream	= $query->execute( $model );
		isa_ok( $stream, 'RDF::Query::Stream' );
		my $current	= $stream->current;
		isa_ok( $current, 'ARRAY' );
		my ($p, $h)	= @{ $current };
		ok( $query->bridge->is_resource( $h ), 'Got a resource for homepage' );
		is( $query->bridge->uri_value( $h ), 'http://www.realify.com/~gary/', 'Got homepage' );
	}
	
	SKIP: {
		print "# datatyped literal\n";
		skip "Redland datatyped literals are broken", 4 if UNIVERSAL::isa($model, 'RDF::Redland::Model');
		my $query	= new RDF::Query ( <<'END', undef, undef, 'sparql' );
			PREFIX	foaf: <http://xmlns.com/foaf/0.1/>
			PREFIX	dc: <http://purl.org/dc/elements/1.1/>
			SELECT	?image
			WHERE	{
						?image dc:date "2005-04-07T18:27:56-04:00"^^<http://www.w3.org/2001/XMLSchema#dateTime>
					}
END
		my $stream	= $query->execute( $model );
		isa_ok( $stream, 'RDF::Query::Stream' );
		my $current	= $stream->current;
		isa_ok( $current, 'ARRAY' );
		my ($h)	= @{ $current };
		ok( $query->bridge->is_resource( $h ), 'Got a resource for image' );
		is( $query->bridge->uri_value( $h ), 'http://kasei.us/pictures/2005/20050422-WCCS_Dinner/images/DSC_8057.jpg', 'Got image by typed date' );
	}
}
